@@ -15,8 +15,9 @@ from simba.features_scripts.unit_tests import (read_video_info_csv,
 from simba.misc_tools import (check_multi_animal_status,
                               SimbaTimer)
 from simba.read_config_unit_tests import (read_config_entry,
-                                          check_that_column_exist,
-                                          read_config_file)
+                                          read_config_file,
+                                          read_project_path_and_file_type)
+from simba.enums import Paths, ReadConfig, Keys, Dtypes
 
 class ROIAnalyzer(object):
     """
@@ -50,21 +51,20 @@ class ROIAnalyzer(object):
 
     def __init__(self,
                  ini_path: str,
-                 data_path: str,
+                 data_path: str or None,
                  calculate_distances: bool = False):
 
         self.timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
         config = read_config_file(ini_path)
-        self.no_animals = read_config_entry(config, 'General settings', 'animal_no', 'int')
-        self.project_path = read_config_entry(config, 'General settings', 'project_path', data_type='folder_path')
+        self.project_path, self.file_type = read_project_path_and_file_type(config=config)
+        self.no_animals = read_config_entry(config, ReadConfig.GENERAL_SETTINGS.value, ReadConfig.ANIMAL_CNT.value, Dtypes.INT.value)
         self.logs_path = os.path.join(self.project_path, 'logs')
-        self.detailed_ROI_data_path = os.path.join(self.project_path, 'logs', 'Detailed_ROI_data')
-        self.vid_info_df = read_video_info_csv(os.path.join(self.logs_path, 'video_info.csv'))
-        self.file_type = read_config_entry(config, 'General settings', 'workflow_file_type', 'str', 'csv')
+        self.detailed_ROI_data_path = os.path.join(self.project_path, Paths.DETAILED_ROI_DATA_DIR.value)
+        self.vid_info_df = read_video_info_csv(os.path.join(self.logs_path, ReadConfig.VIDEO_INFO_CSV.value))
         self.calculate_distances = calculate_distances
         self.bp_headers = getBpHeaders(ini_path)
         if not os.path.exists(self.detailed_ROI_data_path): os.makedirs(self.detailed_ROI_data_path)
-        self.p_thresh = read_config_entry(config, 'ROI settings', 'probability_threshold', 'float', 0.00)
+        self.p_thresh = read_config_entry(config, ReadConfig.ROI_SETTINGS.value, ReadConfig.PROBABILITY_THRESHOLD.value, Dtypes.FLOAT.value, 0.00)
         if data_path != None:
             self.input_folder = os.path.join(self.project_path, 'csv', data_path)
             self.files_found = glob.glob(self.input_folder + '/*.' + self.file_type)
@@ -104,13 +104,13 @@ class ROIAnalyzer(object):
         """
 
 
-        if not os.path.isfile(os.path.join(self.logs_path, 'measures', 'ROI_definitions.h5')):
+        if not os.path.isfile(os.path.join(self.logs_path, Paths.ROI_DEFINITIONS.value)):
             raise FileNotFoundError(print('No ROI definitions were found in your SimBA project. Please draw some ROIs before analyzing your ROI data'))
         else:
-            self.roi_h5_path = os.path.join(self.logs_path, 'measures', 'ROI_definitions.h5')
-            self.rectangles_df = pd.read_hdf(self.roi_h5_path, key='rectangles')
-            self.circles_df = pd.read_hdf(self.roi_h5_path, key='circleDf')
-            self.polygon_df = pd.read_hdf(self.roi_h5_path, key='polygons')
+            self.roi_h5_path = os.path.join(self.logs_path, Paths.ROI_DEFINITIONS.value)
+            self.rectangles_df = pd.read_hdf(self.roi_h5_path, key=Keys.ROI_RECTANGLES.value).dropna(how='any')
+            self.circles_df = pd.read_hdf(self.roi_h5_path, key=Keys.ROI_CIRCLES.value).dropna(how='any')
+            self.polygon_df = pd.read_hdf(self.roi_h5_path, key=Keys.ROI_POLYGONS.value).dropna(how='any')
             self.shape_names = list(itertools.chain(self.rectangles_df['Name'].unique(), self.circles_df['Name'].unique(), self.polygon_df['Name'].unique()))
             self.__reset_dicts()
 
@@ -276,7 +276,6 @@ class ROIAnalyzer(object):
         -------
         None
         """
-
         for data_name, data in zip(['Detailed_ROI_data', 'ROI_time_data', 'ROI_entry_data'], [self.entry_exit_df_lst, self.time_df_lst, self.entry_df_lst]):
             save_df = pd.concat(data, axis=0).reset_index(drop=True)
             save_path = os.path.join(self.logs_path, data_name + '_' + self.timestamp + '.csv')
@@ -291,7 +290,7 @@ class ROIAnalyzer(object):
         self.timer.stop_timer()
         print('SIMBA COMPLETE: ROI analysis complete (elapsed time: {}s)'.format(self.timer.elapsed_time_str))
 
-# test = ROIAnalyzer(ini_path = r"/Users/simon/Desktop/troubleshooting/train_model_project/project_folder/project_config.ini",
+# test = ROIAnalyzer(ini_path = r"/Users/simon/Desktop/envs/troubleshooting/Termites_5/project_folder/project_config.ini",
 #                    data_path = "outlier_corrected_movement_location", calculate_distances=True)
 # test.read_roi_dfs()
 # test.analyze_ROIs()
