@@ -3,10 +3,12 @@ __author__ = "Simon Nilsson", "JJ Choong"
 
 from datetime import datetime
 from simba.read_config_unit_tests import (read_config_entry,
-                                          read_config_file)
+                                          read_config_file,
+                                          read_project_path_and_file_type)
 from simba.misc_tools import SimbaTimer
 import pandas as pd
-import os, glob
+import os
+from simba.enums import ReadConfig
 from simba.ROI_analyzer import ROIAnalyzer
 
 class ROIMovementAnalyzer(object):
@@ -35,8 +37,7 @@ class ROIMovementAnalyzer(object):
         self.timer.start_timer()
         self.config = read_config_file(config_path)
         self.datetime = datetime.now().strftime('%Y%m%d%H%M%S')
-        self.project_path = read_config_entry(self.config, 'General settings', 'project_path', data_type='folder_path')
-        self.file_type = read_config_entry(self.config, 'General settings', 'workflow_file_type', 'str', 'csv')
+        self.project_path, self.file_type = read_project_path_and_file_type(config=self.config)
         self.roi_analyzer = ROIAnalyzer(ini_path=config_path, data_path='outlier_corrected_movement_location', calculate_distances=True)
         self.roi_analyzer.read_roi_dfs()
         self.roi_analyzer.analyze_ROIs()
