@@ -7,6 +7,7 @@ import pandas as pd
 import os
 import itertools
 import cv2
+from simba.enums import Paths, Formats
 from simba.misc_tools import (get_video_meta_data,
                               add_missing_ROI_cols,
                               SimbaTimer)
@@ -38,7 +39,8 @@ class ROIPlot(object):
     def __init__(
             self,
             ini_path: str,
-            video_path: str):
+            video_path: str,
+            style_attr: dict):
 
         self.roi_analyzer = ROIAnalyzer(ini_path=ini_path, data_path='outlier_corrected_movement_location')
         self.video_dir_path = os.path.join(self.roi_analyzer.project_path, 'videos')
@@ -47,15 +49,15 @@ class ROIPlot(object):
         _, self.video_name, _ = get_fn_ext(video_path)
         self.roi_analyzer.files_found = [os.path.join(self.roi_analyzer.input_folder, self.video_name + '.' + self.roi_analyzer.file_type)]
         if not os.path.isfile(self.roi_analyzer.files_found[0]):
-            print('Could not find the file at path {}. Please make sure you have corrected body-part outliers or indicated that you want to skip outlier correction'.format(self.roi_analyzer.files_found[0]))
-            raise FileNotFoundError
+            print('SIMBA ERROR: Could not find the file at path {}. Please make sure you have corrected body-part outliers or indicated that you want to skip outlier correction'.format(self.roi_analyzer.files_found[0]))
+            raise FileNotFoundError()
         self.roi_analyzer.analyze_ROIs()
         self.roi_entries_df = pd.concat(self.roi_analyzer.entry_exit_df_lst, axis=0)
-        self.data_df = self.roi_analyzer.data_df
+        self.data_df, self.style_attr = self.roi_analyzer.data_df, style_attr
         self.video_shapes = list(itertools.chain(self.roi_analyzer.video_recs['Name'].unique(), self.roi_analyzer.video_circs['Name'].unique(),self.roi_analyzer.video_polys['Name'].unique()))
         if len(list(set(self.video_shapes))) != len(self.video_shapes):
             print('SIMBA ERROR: Some SIMBA ROI shapes have identical names. Please use unique names to visualize ROI data.')
-            raise AttributeError
+            raise AttributeError()
         self.roi_analyzer.video_recs = add_missing_ROI_cols(self.roi_analyzer.video_recs)
         self.roi_analyzer.video_circs = add_missing_ROI_cols(self.roi_analyzer.video_circs)
         self.roi_analyzer.video_polys = add_missing_ROI_cols(self.roi_analyzer.video_polys)
@@ -74,7 +76,7 @@ class ROIPlot(object):
                 self.data_df[animal + '_' + shape_name] = 0
                 self.shape_columns.append(animal + '_' + shape_name)
         self.bp_dict = self.roi_analyzer.bp_dict
-        self.output_folder = os.path.join(self.roi_analyzer.project_path, 'frames', 'output', 'ROI_analysis')
+        self.output_folder = os.path.join(self.roi_analyzer.project_path, Paths.ROI_ANALYSIS.value)
         if not os.path.exists(self.output_folder):
             os.makedirs(self.output_folder)
         self.timer = SimbaTimer()
@@ -149,9 +151,6 @@ class ROIPlot(object):
         None
         """
 
-
-
-
         save_path = os.path.join(self.output_folder, self.video_name + '.avi')
         self.cap = cv2.VideoCapture(self.video_path)
         self.video_meta_data = get_video_meta_data(self.video_path)
@@ -160,7 +159,7 @@ class ROIPlot(object):
         max_dim = max(self.video_meta_data['width'], self.video_meta_data['height'])
         self.font = cv2.FONT_HERSHEY_TRIPLEX
         draw_scale, self.font_size = int(radius_scale / (res_scale / max_dim)), float(font_scale / (res_scale / max_dim))
-        fourcc = cv2.VideoWriter_fourcc(*'XVID')
+        fourcc = cv2.VideoWriter_fourcc(*Formats.AVI_CODEC.value)
         self.spacing_scaler = int(self.space_scale / (res_scale / max_dim))
         writer = cv2.VideoWriter(save_path, fourcc, fps, (self.video_meta_data['width'] * 2, self.video_meta_data['height']))
         color_lst = createColorListofList(self.roi_analyzer.no_animals, int((len(self.roi_analyzer.bp_columns) / 3)))[0]
@@ -180,10 +179,7 @@ class ROIPlot(object):
                     self.__insert_texts(self.roi_analyzer.video_polys)
 
                     for _, row in self.roi_analyzer.video_recs.iterrows():
-
                         top_left_x, top_left_y, shape_name = row['topLeftX'], row['topLeftY'], row['Name']
-                        # if shape_name == 'Upper_compartment_bottom_left':
-                        #     print(print(row))
                         bottom_right_x, bottom_right_y = row['Bottom_right_X'], row['Bottom_right_Y']
                         thickness, color = row['Thickness'], row['Color BGR']
                         cv2.rectangle(self.border_img, (top_left_x, top_left_y), (bottom_right_x, bottom_right_y), color, thickness)
@@ -201,8 +197,10 @@ class ROIPlot(object):
                     for animal_cnt, animal_name in enumerate(self.roi_analyzer.multiAnimalIDList):
                         bp_data = self.data_df.loc[frame_cnt, self.bp_dict[animal_name]].values
                         if self.roi_analyzer.p_thresh < bp_data[2]:
-                            cv2.circle(self.border_img, (int(bp_data[0]), int(bp_data[1])), draw_scale, color_lst[animal_cnt], -1)
-                            cv2.putText(self.border_img, animal_name, (int(bp_data[0]), int(bp_data[1])), self.font, self.font_size, color_lst[animal_cnt], 1)
+                            if self.style_attr['Show_body_part']:
+                                cv2.circle(self.border_img, (int(bp_data[0]), int(bp_data[1])), draw_scale, color_lst[animal_cnt], -1)
+                            if self.style_attr['Show_animal_name']:
+                                cv2.putText(self.border_img, animal_name, (int(bp_data[0]), int(bp_data[1])), self.font, self.font_size, color_lst[animal_cnt], 1)
                             for shape_name in self.video_shapes:
                                 if self.data_df.loc[frame_cnt, animal_name + '_' + shape_name] == 1:
                                     self.cnt_dict[animal_name][shape_name]['timer'] += (1 / fps)
@@ -237,7 +235,7 @@ class ROIPlot(object):
         writer.release()
 
 
-# test = ROIPlot(ini_path=r'/Users/simon/Desktop/troubleshooting/B1-MS_US/project_folder/project_config.ini', video_path="CD1-B1-Ph1-T2-MS_US10min.mp4")
+# test = ROIPlot(ini_path=r'/Users/simon/Desktop/envs/troubleshooting/Termites_5/project_folder/project_config.ini', video_path="termite_test.mp4")
 # test.insert_data()
 # test.visualize_ROI_data()
 
