@@ -12,8 +12,13 @@ from datetime import datetime
 from simba.misc_tools import (get_fn_ext,
                               detect_bouts,
                               SimbaTimer)
-from simba.read_config_unit_tests import read_config_file, read_config_entry, check_that_column_exist
-from simba.features_scripts.unit_tests import read_video_info_csv, read_video_info
+from simba.read_config_unit_tests import (read_config_file,
+                                          read_config_entry,
+                                          check_that_column_exist,
+                                          check_if_filepath_list_is_empty)
+from simba.features_scripts.unit_tests import (read_video_info_csv,
+                                               read_video_info)
+from simba.enums import Keys, Paths, ReadConfig, Dtypes
 
 
 class clf_within_ROI(object):
@@ -39,20 +44,20 @@ class clf_within_ROI(object):
                  config_ini: str):
 
         self.config = read_config_file(ini_path=config_ini)
-        self.projectPath = read_config_entry(self.config, 'General settings', 'project_path', data_type='folder_path')
-        model_nos = read_config_entry(config=self.config, section='SML settings', option='No_targets', data_type='int')
+        self.projectPath = read_config_entry(self.config, ReadConfig.GENERAL_SETTINGS.value, ReadConfig.PROJECT_PATH.value, data_type=ReadConfig.FOLDER_PATH.value)
+        model_nos = read_config_entry(config=self.config, section=ReadConfig.SML_SETTINGS.value, option=ReadConfig.TARGET_CNT.value, data_type=Dtypes.INT.value)
         self.log_path_dir = os.path.join(self.projectPath, 'logs')
         self.video_info_df = read_video_info_csv(os.path.join(self.log_path_dir, 'video_info.csv'))
-        self.body_parts_path = os.path.join(os.path.join(self.log_path_dir, 'measures', 'pose_configs', 'bp_names', 'project_bp_names.csv'))
+        self.body_parts_path = os.path.join(self.projectPath, Paths.BP_NAMES.value)
         body_parts_df = pd.read_csv(self.body_parts_path, names=['bodyparts'])
         self.body_part_list = list(body_parts_df['bodyparts'])
-        self.ROIcoordinatesPath = os.path.join(self.log_path_dir, 'measures', 'ROI_definitions.h5')
+        self.ROIcoordinatesPath = os.path.join(self.log_path_dir, Paths.ROI_DEFINITIONS.value)
         if not os.path.isfile(self.ROIcoordinatesPath):
             print('No ROI coordinates found. Please use the [ROI] tab to define ROIs')
             raise FileNotFoundError()
-        self.rectangles_df = pd.read_hdf(self.ROIcoordinatesPath, key='rectangles')
-        self.circles_df = pd.read_hdf(self.ROIcoordinatesPath, key='circleDf')
-        self.polygon_df = pd.read_hdf(self.ROIcoordinatesPath, key='polygons')
+        self.rectangles_df = pd.read_hdf(self.ROIcoordinatesPath, key=Keys.ROI_RECTANGLES.value)
+        self.circles_df = pd.read_hdf(self.ROIcoordinatesPath, key=Keys.ROI_CIRCLES.value)
+        self.polygon_df = pd.read_hdf(self.ROIcoordinatesPath, key=Keys.ROI_POLYGONS.value)
         self.ROI_str_name_list = []
         for index, row in self.rectangles_df.iterrows(): self.ROI_str_name_list.append(row['Shape_type'] + ': ' + row['Name'])
         for index, row in self.circles_df.iterrows(): self.ROI_str_name_list.append(row['Shape_type'] + ': ' + row['Name'])
@@ -60,7 +65,7 @@ class clf_within_ROI(object):
         self.ROI_str_name_list = list(set(self.ROI_str_name_list))
         self.behavior_names = []
         for i in range(model_nos):
-            self.behavior_names.append(self.config.get('SML settings', 'target_name_' + str(i + 1)))
+            self.behavior_names.append(self.config.get(ReadConfig.SML_SETTINGS.value, 'target_name_' + str(i + 1)))
 
     def __inside_rectangle(self, bp_x, bp_y, top_left_x, top_left_y, bottom_right_x, bottom_right_y):
         """
@@ -154,12 +159,11 @@ class clf_within_ROI(object):
         self.ROI_dict_lists, self.behavior_list, self.measurements = ROI_dict_lists, behavior_list, measurements
         self.file_type = read_config_entry(config=self.config, section='General settings', option='workflow_file_type', data_type='str')
         files_found = glob.glob(machine_results_path + '/*.' + self.file_type)
-        if len(files_found) == 0:
-            print('SIMBA ERROR: No machine learning results found in the project_folder/csv/machine_results directory. Create machine classifications before analyzing classifications by ROI')
-            raise ValueError()
+        check_if_filepath_list_is_empty(filepaths=files_found,
+                                        error_msg='SIMBA ERROR: No machine learning results found in the project_folder/csv/machine_results directory. Create machine classifications before analyzing classifications by ROI')
         if len(behavior_list) == 0:
             print('SIMBA ERROR: Please select at least one classifier')
-            raise ValueError()
+            raise ValueError('SIMBA ERROR: Please select at least one classifier')
         print('Analyzing {} files...'.format(str(len(files_found))))
         body_part_col_names = []
         body_part_col_names_x, body_part_col_names_y = [], []
@@ -242,105 +246,4 @@ class clf_within_ROI(object):
         print('SIMBA COMPLETE: Classification data by ROIs saved in {} (selapsed time: {}s)'.format(out_path, self.timer.elapsed_time_str))
 
 # clf_ROI_analyzer = clf_within_ROI(config_ini="/Users/simon/Desktop/troubleshooting/train_model_project/project_folder/project_config.ini")
-# clf_ROI_analyzer.perform_ROI_clf_analysis(behavior_list=['Attack', 'Sniffing'], ROI_dict_lists={'Rectangle': ['rec'], 'Circle': ['Stimulus 1', 'Stimulus 2', 'Stimulus 3']}, body_part_list=['Nose_1'], measurements=['Total time by ROI (s)', 'Started bouts by ROI (count)', 'Ended bouts by ROI (count)'])
-
-        #  =
-                        #
-                        # #                             current_behavior_df['topLeftY'] = topLeftY
-                        #
-                        #
-                        #
-    #                         if
-    #                             pass
-    #                         else:
-    #                             topLeftX, topLeftY = current_rectangle_info['topLeftX'].values[0], current_rectangle_info['topLeftY'].values[0]
-    #                             bottomRightX, bottomRightY = topLeftX + current_rectangle_info['width'].values[0], topLeftY + + current_rectangle_info['height'].values[0]
-    #                             current_behavior_df['topLeftX'] = topLeftX
-    #                             current_behavior_df['topLeftY'] = topLeftY
-    #                             current_behavior_df['bottomRightX'] = bottomRightX
-    #                             current_behavior_df['bottomRightY'] = bottomRightY
-    #
-    #
-    #                         current_circle_info = self.circleInfo.loc[(self.circleInfo['Video'] == self.video_name) & ( self.circleInfo['Shape_type'] == current_type) & (self.circleInfo['Name'] == self.current_name)]
-    #                         if len(current_circle_info) == 0:
-    #                             pass
-    #                         else:
-    #                             centre_x, centre_y = current_circle_info['centerX'].values[0], current_circle_info['centerY'].values[0]
-    #                             radius = current_circle_info['radius'].values[0]
-    #                             current_behavior_df['centerX'] = centre_x
-    #                             current_behavior_df['centerY'] = centre_y
-    #                             current_behavior_df['radius'] = radius
-    #                             current_behavior_df.apply(lambda row: self.__inside_circle(row[body_part_col_names_x[0]],
-    #                                                                                         row[body_part_col_names_y[0]],
-    #                                                                                         row['centerX'],
-    #                                                                                         row['centerY'],
-    #                                                                                         row['radius']), axis=1)
-    #                     if current_type.lower() == 'polygon':
-    #                         current_polygon_info = self.polygonInfo.loc[(self.polygonInfo['Video'] == self.video_name) & (self.polygonInfo['Shape_type'] == current_type) & (self.polygonInfo['Name'] == self.current_name)]
-    #                         if len(current_polygon_info) == 0:
-    #                             pass
-    #                         else:
-    #                             vertices = current_polygon_info['vertices'].values[0]
-    #                             polygon_shape = []
-    #                             for i in vertices:
-    #                                 polygon_shape.append(geometry.Point(i))
-    #                             current_behavior_df.apply(lambda row: self.__inside_polygon(row[body_part_col_names_x[0]], row[body_part_col_names_y[0]], polygon_shape), axis=1)
-    #
-    #     self.__organize_output_data(self.frame_counter_dict)
-    #
-    # def __inside_rectangle(self, bp_x, bp_y, top_left_x, top_left_y, bottom_right_x, bottom_right_y):
-    #     self.__populate_dict()
-    #     if (((top_left_x - 10) <= bp_x <= (bottom_right_x + 10)) and ((top_left_y - 10) <= bp_y <= (bottom_right_y + 10))):
-    #         self.__add_to_counter()
-    #     else:
-    #         pass
-    #
-    # def __inside_circle(self, bp_x, bp_y, center_x, center_y, radius):
-    #     self.__populate_dict()
-    #     px_dist = int(np.sqrt((bp_x - center_x) ** 2 + (bp_y - center_y) ** 2))
-    #     if px_dist <= radius:
-    #         self.__add_to_counter()
-    #     else:
-    #         pass
-    #
-    # def __inside_polygon(self, bp_x, bp_y, vertices):
-    #     self.__populate_dict()
-    #     current_point = Point(int(bp_x), int(bp_y))
-    #     current_polygon = geometry.Polygon([[p.x, p.y] for p in vertices])
-    #     polygon_status = current_polygon.contains(current_point)
-    #     if polygon_status:
-    #         self.__add_to_counter()
-    #     else:
-    #         pass
-    #
-    # def __populate_dict(self):
-    #     if not self.video_name in self.frame_counter_dict:
-    #         self.frame_counter_dict[self.video_name] = {}
-    #     if not self.current_name in self.frame_counter_dict[self.video_name]:
-    #         self.frame_counter_dict[self.video_name][self.current_name] = {}
-    #     if not self.current_behavior in self.frame_counter_dict[self.video_name][self.current_name]:
-    #         self.frame_counter_dict[self.video_name][self.current_name][self.current_behavior] = 0
-    #
-    # def __add_to_counter(self):
-    #     self.frame_counter_dict[self.video_name][self.current_name][self.current_behavior] += 1 / self.fps
-    #
-    # def __organize_output_data(self, frame_counter_dict):
-    #     if len(frame_counter_dict.keys()) == 0:
-    #         print('SIMBA ERROR: ZERO ROIs found the videos represented in the project_folder/csv/machine_results directory')
-    #         raise ValueError('SIMBA ERROR: ZERO ROIs found the videos represented in the project_folder/csv/machine_results directory')
-    #     out_df = pd.concat({k: pd.DataFrame(v) for k, v in frame_counter_dict.items()}, axis=1).T.reset_index()
-    #     out_df = out_df.rename(columns={'level_0': 'Video', 'level_1': 'ROI'})
-    #     out_df = out_df.fillna(0)
-    #     time_columns = list(out_df.columns)
-    #     time_columns.remove('Video')
-    #     time_columns.remove('ROI')
-    #     for column in time_columns:
-    #         out_df = out_df.round({column: 3})
-    #         out_df = out_df.rename(columns={column: str(column) + ' (s)'})
-    #     dateTime = datetime.now().strftime('%Y%m%d%H%M%S')
-    #     out_path = os.path.join(self.logFolderPath, 'Classification_time_by_ROI_' + str(dateTime) + '.csv')
-    #     out_df.to_csv(out_path)
-    #     print('All videos analysed.')
-    #     print('Data saved @ ' + out_path)
-    #
-
+# clf_ROI_analyzer.perform_ROI_clf_analysis(behavior_list=['Attack', 'Sniffing'], ROI_dict_lists={'Rectangle': ['rec'], 'Circle': ['Stimulus 1', 'Stimulus 2', 'Stimulus 3']}, body_part_list=['Nose_1'], measurements=['Total time by ROI (s)', 'Started bouts by ROI (count)', 'Ended bouts by ROI (count)'])
\ No newline at end of file
