@@ -1,11 +1,11 @@
-from simba.read_config_unit_tests import (read_config_entry, check_int, check_str, insert_default_headers_for_feature_extraction, check_file_exist_and_readable, read_config_file)
-import os, glob
-from simba.features_scripts.unit_tests import read_video_info_csv, read_video_info, check_minimum_roll_windows
+from simba.read_config_unit_tests import insert_default_headers_for_feature_extraction
+import os
+from simba.features_scripts.unit_tests import (read_video_info)
+from simba.misc_tools import get_feature_extraction_headers
 from simba.drop_bp_cords import get_fn_ext
 from simba.rw_dfs import read_df, save_df
 import pandas as pd
 import numpy as np
-from numba import jit, prange
 from copy import deepcopy
 import math
 from scipy.spatial import ConvexHull
@@ -13,8 +13,9 @@ from collections import defaultdict
 import scipy
 from joblib import Parallel, delayed
 import time
+from simba.features_scripts.feature_extraction_mixin import FeatureExtractionMixin
 
-class ExtractFeaturesFrom16bps(object):
+class ExtractFeaturesFrom16bps(FeatureExtractionMixin):
     """
     Class for creating a hard-coded set of features from two animals with 8 tracked body-parts
     each using pose-estimation. Results are stored in the `project_folder/csv/features_extracted`
@@ -37,26 +38,8 @@ class ExtractFeaturesFrom16bps(object):
     def __init__(self,
                  config_path: str):
 
-        self.config = read_config_file(config_path)
-        self.project_path = read_config_entry(self.config, 'General settings', 'project_path', data_type='folder_path')
-        self.data_in_dir = os.path.join(self.project_path, 'csv', 'outlier_corrected_movement_location')
-        self.save_dir = os.path.join(self.project_path, 'csv', 'features_extracted')
-        if not os.path.exists(self.save_dir): os.makedirs(self.save_dir)
-        self.vid_info_df = read_video_info_csv(os.path.join(self.project_path, 'logs', 'video_info.csv'))
-        self.roll_windows_values = check_minimum_roll_windows([2, 5, 6, 7.5, 15], self.vid_info_df['fps'].min())
-        self.file_type = read_config_entry(self.config, 'General settings', 'workflow_file_type', 'str', 'csv')
-        self.files_found = glob.glob(self.data_in_dir + '/*.' + self.file_type)
-        self.in_headers = ["Ear_left_1_x", "Ear_left_1_y", "Ear_left_1_p", "Ear_right_1_x", "Ear_right_1_y",
-                         "Ear_right_1_p", "Nose_1_x", "Nose_1_y", "Nose_1_p", "Center_1_x", "Center_1_y", "Center_1_p",
-                         "Lat_left_1_x", "Lat_left_1_y",
-                         "Lat_left_1_p", "Lat_right_1_x", "Lat_right_1_y", "Lat_right_1_p", "Tail_base_1_x",
-                         "Tail_base_1_y", "Tail_base_1_p", "Tail_end_1_x", "Tail_end_1_y", "Tail_end_1_p",
-                         "Ear_left_2_x",
-                         "Ear_left_2_y", "Ear_left_2_p", "Ear_right_2_x", "Ear_right_2_y", "Ear_right_2_p",
-                         "Nose_2_x", "Nose_2_y", "Nose_2_p", "Center_2_x", "Center_2_y", "Center_2_p", "Lat_left_2_x",
-                         "Lat_left_2_y",
-                         "Lat_left_2_p", "Lat_right_2_x", "Lat_right_2_y", "Lat_right_2_p", "Tail_base_2_x",
-                         "Tail_base_2_y", "Tail_base_2_p", "Tail_end_2_x", "Tail_end_2_y", "Tail_end_2_p"]
+        super().__init__(config_path=config_path)
+        self.in_headers = get_feature_extraction_headers(pose='2 animals 16 body-parts')
         self.mouse_1_headers, self.mouse_2_headers = self.in_headers[0:24], self.in_headers[24:]
         self.mouse_2_p_headers = [x for x in self.mouse_2_headers if x[-2:] == '_p']
         self.mouse_1_p_headers = [x for x in self.mouse_1_headers if x[-2:] == '_p']
@@ -64,56 +47,6 @@ class ExtractFeaturesFrom16bps(object):
         self.mouse_2_headers = [x for x in self.mouse_2_headers if x[-2:] != '_p']
         print('Extracting features from {} file(s)...'.format(str(len(self.files_found))))
 
-    @staticmethod
-    @jit(nopython=True)
-    def __euclidean_distance(bp_1_x_vals, bp_2_x_vals, bp_1_y_vals, bp_2_y_vals, px_per_mm):
-        series = (np.sqrt((bp_1_x_vals - bp_2_x_vals) ** 2 + (bp_1_y_vals - bp_2_y_vals) ** 2)) / px_per_mm
-        return series
-
-    @staticmethod
-    @jit(nopython=True, fastmath=True)
-    def __angle3pt(ax, ay, bx, by, cx, cy):
-        ang = math.degrees(math.atan2(cy - by, cx - bx) - math.atan2(ay - by, ax - bx))
-        return ang + 360 if ang < 0 else ang
-
-    @staticmethod
-    @jit(nopython=True, fastmath=True)
-    def __angle3pt_serialized(data: np.array):
-        results = np.full((data.shape[0]), 0.0)
-        for i in prange(data.shape[0]):
-            angle = math.degrees(math.atan2(data[i][5] - data[i][3], data[i][4] - data[i][2]) - math.atan2(data[i][1] - data[i][3], data[i][0] - data[i][2]))
-            if angle < 0:
-                angle += 360
-            results[i] = angle
-
-        return results
-
-
-    @staticmethod
-    def convex_hull_calculator_mp(arr: np.array, px_per_mm: float) -> float:
-        arr = np.unique(arr, axis=0)
-        if arr.shape[0] < 3:
-            return 0
-        for i in range(1, arr.shape[0]):
-            if (arr[i] != arr[0]).all():
-                return ConvexHull(arr).area / px_per_mm
-            else:
-                pass
-        return 0
-
-
-    @staticmethod
-    @jit(nopython=True)
-    def __count_values_in_range(data: np.array, ranges: np.array):
-        results = np.full((data.shape[0], ranges.shape[0]), 0)
-        for i in prange(data.shape[0]):
-            for j in prange(ranges.shape[0]):
-                lower_bound, upper_bound = ranges[j][0], ranges[j][1]
-                results[i][j] = data[i][np.logical_and(data[i] >= lower_bound, data[i] <= upper_bound)].shape[0]
-        return results
-
-
-
     def extract_features(self):
         """
         Method to compute and save feature battery to disk. Results are saved in the `project_folder/csv/features_extracted`
@@ -140,46 +73,46 @@ class ExtractFeaturesFrom16bps(object):
             self.out_data['Mouse_2_poly_area'] = Parallel(n_jobs=-1, verbose=0, backend="threading")(delayed(self.convex_hull_calculator_mp)(x, self.px_per_mm) for x in mouse_2_ar)
             self.in_data_shifted = self.out_data.shift(periods=1).add_suffix('_shifted').fillna(0)
             self.in_data = pd.concat([self.in_data, self.in_data_shifted], axis=1, join='inner').fillna(0).reset_index(drop=True)
-            self.out_data['Mouse_1_nose_to_tail'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Tail_base_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Tail_base_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_nose_to_tail'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Tail_base_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Tail_base_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_width'] = self.__euclidean_distance(self.out_data['Lat_left_1_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Lat_left_1_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_width'] = self.__euclidean_distance(self.out_data['Lat_left_2_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Lat_left_2_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_Ear_distance'] = self.__euclidean_distance(self.out_data['Ear_left_1_x'].values, self.out_data['Ear_right_1_x'].values, self.out_data['Ear_left_1_y'].values, self.out_data['Ear_right_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_Ear_distance'] = self.__euclidean_distance(self.out_data['Ear_left_2_x'].values, self.out_data['Ear_right_2_x'].values, self.out_data['Ear_left_2_y'].values, self.out_data['Ear_right_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_Nose_to_centroid'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Center_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Center_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_Nose_to_centroid'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Center_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Center_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_Nose_to_lateral_left'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_left_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_left_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_Nose_to_lateral_left'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_left_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_left_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_Nose_to_lateral_right'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_Nose_to_lateral_right'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_Centroid_to_lateral_left'] = self.__euclidean_distance(self.out_data['Center_1_x'].values, self.out_data['Lat_left_1_x'].values, self.out_data['Center_1_y'].values, self.out_data['Lat_left_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_Centroid_to_lateral_left'] = self.__euclidean_distance(self.out_data['Center_2_x'].values, self.out_data['Lat_left_2_x'].values, self.out_data['Center_2_y'].values, self.out_data['Lat_left_2_y'].values, self.px_per_mm)
-            self.out_data['Mouse_1_Centroid_to_lateral_right'] = self.__euclidean_distance(self.out_data['Center_1_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Center_1_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
-            self.out_data['Mouse_2_Centroid_to_lateral_right'] = self.__euclidean_distance(self.out_data['Center_2_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Center_2_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
-            self.out_data['Centroid_distance'] = self.__euclidean_distance(self.out_data['Center_2_x'].values, self.out_data['Center_1_x'].values, self.out_data['Center_2_y'].values, self.out_data['Center_1_y'].values, self.px_per_mm)
-            self.out_data['Nose_to_nose_distance'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Nose_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Nose_1_y'].values, self.px_per_mm)
-            self.out_data['M1_Nose_to_M2_lat_left'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_left_2_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_left_2_y'].values, self.px_per_mm)
-            self.out_data['M1_Nose_to_M2_lat_right'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
-            self.out_data['M2_Nose_to_M1_lat_left'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_left_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_left_1_y'].values, self.px_per_mm)
-            self.out_data['M2_Nose_to_M1_lat_right'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
-            self.out_data['M1_Nose_to_M2_tail_base'] = self.__euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Tail_base_2_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Tail_base_2_y'].values, self.px_per_mm)
-            self.out_data['M2_Nose_to_M1_tail_base'] = self.__euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Tail_base_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Tail_base_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_centroid'] = self.__euclidean_distance(self.in_data['Center_1_x_shifted'].values, self.in_data['Center_1_x'].values, self.in_data['Center_1_y_shifted'].values, self.in_data['Center_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_centroid'] = self.__euclidean_distance(self.in_data['Center_2_x_shifted'].values, self.in_data['Center_2_x'].values, self.in_data['Center_2_y_shifted'].values, self.in_data['Center_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_nose'] = self.__euclidean_distance(self.in_data['Nose_1_x_shifted'].values, self.in_data['Nose_1_x'].values, self.in_data['Nose_1_y_shifted'].values, self.in_data['Nose_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_nose'] = self.__euclidean_distance(self.in_data['Nose_2_x_shifted'].values, self.in_data['Nose_2_x'].values, self.in_data['Nose_2_y_shifted'].values, self.in_data['Nose_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_tail_base'] = self.__euclidean_distance(self.in_data['Tail_base_1_x_shifted'].values, self.in_data['Tail_base_1_x'].values, self.in_data['Tail_base_1_y_shifted'].values, self.in_data['Tail_base_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_tail_base'] = self.__euclidean_distance(self.in_data['Tail_base_2_x_shifted'].values, self.in_data['Tail_base_2_x'].values, self.in_data['Tail_base_2_y_shifted'].values, self.in_data['Tail_base_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_tail_end'] = self.__euclidean_distance(self.in_data['Tail_end_1_x_shifted'].values, self.in_data['Tail_end_1_y_shifted'].values, self.in_data['Tail_end_1_x'].values, self.in_data['Tail_end_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_tail_end'] = self.__euclidean_distance(self.in_data['Tail_end_2_x_shifted'].values, self.in_data['Tail_end_2_y_shifted'].values, self.in_data['Tail_end_2_x'].values, self.in_data['Tail_end_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_left_ear'] = self.__euclidean_distance(self.in_data['Ear_left_1_x_shifted'].values, self.in_data['Ear_left_1_x'].values, self.in_data['Ear_left_1_y_shifted'].values, self.in_data['Ear_left_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_left_ear'] = self.__euclidean_distance(self.in_data['Ear_left_2_x_shifted'].values, self.in_data['Ear_left_2_x'].values, self.in_data['Ear_left_2_y_shifted'].values, self.in_data['Ear_left_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_right_ear'] = self.__euclidean_distance(self.in_data['Ear_right_1_x_shifted'].values, self.in_data['Ear_right_1_x'].values, self.in_data['Ear_right_1_y_shifted'].values, self.in_data['Ear_right_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_right_ear'] = self.__euclidean_distance(self.in_data['Ear_right_2_x_shifted'].values, self.in_data['Ear_right_2_x'].values, self.in_data['Ear_right_2_y_shifted'].values, self.in_data['Ear_right_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_lateral_left'] = self.__euclidean_distance(self.in_data['Lat_left_1_x_shifted'].values, self.in_data['Lat_left_1_x'].values, self.in_data['Lat_left_1_y_shifted'].values, self.in_data['Lat_left_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_lateral_left'] = self.__euclidean_distance(self.in_data['Lat_left_2_x_shifted'].values, self.in_data['Lat_left_2_x'].values, self.in_data['Lat_left_2_y_shifted'].values, self.in_data['Lat_left_2_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_1_lateral_right'] = self.__euclidean_distance(self.in_data['Lat_right_1_x_shifted'].values, self.in_data['Lat_right_1_x'].values, self.in_data['Lat_right_1_y_shifted'].values, self.in_data['Lat_right_1_y'].values, self.px_per_mm)
-            self.out_data['Movement_mouse_2_lateral_right'] = self.__euclidean_distance(self.in_data['Lat_right_2_x_shifted'].values, self.in_data['Lat_right_2_x'].values, self.in_data['Lat_right_2_y_shifted'].values, self.in_data['Lat_right_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_nose_to_tail'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Tail_base_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Tail_base_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_nose_to_tail'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Tail_base_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Tail_base_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_width'] = self.euclidean_distance(self.out_data['Lat_left_1_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Lat_left_1_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_width'] = self.euclidean_distance(self.out_data['Lat_left_2_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Lat_left_2_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_Ear_distance'] = self.euclidean_distance(self.out_data['Ear_left_1_x'].values, self.out_data['Ear_right_1_x'].values, self.out_data['Ear_left_1_y'].values, self.out_data['Ear_right_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_Ear_distance'] = self.euclidean_distance(self.out_data['Ear_left_2_x'].values, self.out_data['Ear_right_2_x'].values, self.out_data['Ear_left_2_y'].values, self.out_data['Ear_right_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_Nose_to_centroid'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Center_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Center_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_Nose_to_centroid'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Center_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Center_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_Nose_to_lateral_left'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_left_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_left_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_Nose_to_lateral_left'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_left_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_left_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_Nose_to_lateral_right'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_Nose_to_lateral_right'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_Centroid_to_lateral_left'] = self.euclidean_distance(self.out_data['Center_1_x'].values, self.out_data['Lat_left_1_x'].values, self.out_data['Center_1_y'].values, self.out_data['Lat_left_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_Centroid_to_lateral_left'] = self.euclidean_distance(self.out_data['Center_2_x'].values, self.out_data['Lat_left_2_x'].values, self.out_data['Center_2_y'].values, self.out_data['Lat_left_2_y'].values, self.px_per_mm)
+            self.out_data['Mouse_1_Centroid_to_lateral_right'] = self.euclidean_distance(self.out_data['Center_1_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Center_1_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
+            self.out_data['Mouse_2_Centroid_to_lateral_right'] = self.euclidean_distance(self.out_data['Center_2_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Center_2_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
+            self.out_data['Centroid_distance'] = self.euclidean_distance(self.out_data['Center_2_x'].values, self.out_data['Center_1_x'].values, self.out_data['Center_2_y'].values, self.out_data['Center_1_y'].values, self.px_per_mm)
+            self.out_data['Nose_to_nose_distance'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Nose_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Nose_1_y'].values, self.px_per_mm)
+            self.out_data['M1_Nose_to_M2_lat_left'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_left_2_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_left_2_y'].values, self.px_per_mm)
+            self.out_data['M1_Nose_to_M2_lat_right'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Lat_right_2_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Lat_right_2_y'].values, self.px_per_mm)
+            self.out_data['M2_Nose_to_M1_lat_left'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_left_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_left_1_y'].values, self.px_per_mm)
+            self.out_data['M2_Nose_to_M1_lat_right'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Lat_right_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Lat_right_1_y'].values, self.px_per_mm)
+            self.out_data['M1_Nose_to_M2_tail_base'] = self.euclidean_distance(self.out_data['Nose_1_x'].values, self.out_data['Tail_base_2_x'].values, self.out_data['Nose_1_y'].values, self.out_data['Tail_base_2_y'].values, self.px_per_mm)
+            self.out_data['M2_Nose_to_M1_tail_base'] = self.euclidean_distance(self.out_data['Nose_2_x'].values, self.out_data['Tail_base_1_x'].values, self.out_data['Nose_2_y'].values, self.out_data['Tail_base_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_centroid'] = self.euclidean_distance(self.in_data['Center_1_x_shifted'].values, self.in_data['Center_1_x'].values, self.in_data['Center_1_y_shifted'].values, self.in_data['Center_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_centroid'] = self.euclidean_distance(self.in_data['Center_2_x_shifted'].values, self.in_data['Center_2_x'].values, self.in_data['Center_2_y_shifted'].values, self.in_data['Center_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_nose'] = self.euclidean_distance(self.in_data['Nose_1_x_shifted'].values, self.in_data['Nose_1_x'].values, self.in_data['Nose_1_y_shifted'].values, self.in_data['Nose_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_nose'] = self.euclidean_distance(self.in_data['Nose_2_x_shifted'].values, self.in_data['Nose_2_x'].values, self.in_data['Nose_2_y_shifted'].values, self.in_data['Nose_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_tail_base'] = self.euclidean_distance(self.in_data['Tail_base_1_x_shifted'].values, self.in_data['Tail_base_1_x'].values, self.in_data['Tail_base_1_y_shifted'].values, self.in_data['Tail_base_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_tail_base'] = self.euclidean_distance(self.in_data['Tail_base_2_x_shifted'].values, self.in_data['Tail_base_2_x'].values, self.in_data['Tail_base_2_y_shifted'].values, self.in_data['Tail_base_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_tail_end'] = self.euclidean_distance(self.in_data['Tail_end_1_x_shifted'].values, self.in_data['Tail_end_1_y_shifted'].values, self.in_data['Tail_end_1_x'].values, self.in_data['Tail_end_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_tail_end'] = self.euclidean_distance(self.in_data['Tail_end_2_x_shifted'].values, self.in_data['Tail_end_2_y_shifted'].values, self.in_data['Tail_end_2_x'].values, self.in_data['Tail_end_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_left_ear'] = self.euclidean_distance(self.in_data['Ear_left_1_x_shifted'].values, self.in_data['Ear_left_1_x'].values, self.in_data['Ear_left_1_y_shifted'].values, self.in_data['Ear_left_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_left_ear'] = self.euclidean_distance(self.in_data['Ear_left_2_x_shifted'].values, self.in_data['Ear_left_2_x'].values, self.in_data['Ear_left_2_y_shifted'].values, self.in_data['Ear_left_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_right_ear'] = self.euclidean_distance(self.in_data['Ear_right_1_x_shifted'].values, self.in_data['Ear_right_1_x'].values, self.in_data['Ear_right_1_y_shifted'].values, self.in_data['Ear_right_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_right_ear'] = self.euclidean_distance(self.in_data['Ear_right_2_x_shifted'].values, self.in_data['Ear_right_2_x'].values, self.in_data['Ear_right_2_y_shifted'].values, self.in_data['Ear_right_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_lateral_left'] = self.euclidean_distance(self.in_data['Lat_left_1_x_shifted'].values, self.in_data['Lat_left_1_x'].values, self.in_data['Lat_left_1_y_shifted'].values, self.in_data['Lat_left_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_lateral_left'] = self.euclidean_distance(self.in_data['Lat_left_2_x_shifted'].values, self.in_data['Lat_left_2_x'].values, self.in_data['Lat_left_2_y_shifted'].values, self.in_data['Lat_left_2_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_1_lateral_right'] = self.euclidean_distance(self.in_data['Lat_right_1_x_shifted'].values, self.in_data['Lat_right_1_x'].values, self.in_data['Lat_right_1_y_shifted'].values, self.in_data['Lat_right_1_y'].values, self.px_per_mm)
+            self.out_data['Movement_mouse_2_lateral_right'] = self.euclidean_distance(self.in_data['Lat_right_2_x_shifted'].values, self.in_data['Lat_right_2_x'].values, self.in_data['Lat_right_2_y_shifted'].values, self.in_data['Lat_right_2_y'].values, self.px_per_mm)
             self.out_data['Mouse_1_polygon_size_change'] = (self.in_data['Mouse_1_poly_area_shifted'] - self.out_data['Mouse_1_poly_area'])
             self.out_data['Mouse_2_polygon_size_change'] = (self.in_data['Mouse_2_poly_area_shifted'] - self.out_data['Mouse_2_poly_area'])
 
@@ -193,7 +126,7 @@ class ExtractFeaturesFrom16bps(object):
                 animal_1_dist, animal_2_dist = animal_1_dist[animal_1_dist != 0], animal_2_dist[animal_2_dist != 0]
                 for animal, animal_name in zip([animal_1_dist, animal_2_dist], ['M1', 'M2']):
                     self.hull_dict['{}_hull_large_euclidean'.format(animal_name)].append(np.amax(animal, initial=0) / self.px_per_mm)
-                    self.hull_dict['{}_hull_small_euclidean'.format(animal_name)].append(np.min(animal, initial=0) / self.px_per_mm)
+                    self.hull_dict['{}_hull_small_euclidean'.format(animal_name)].append(np.min(animal, initial=self.hull_dict['{}_hull_large_euclidean'.format(animal_name)][-1]) / self.px_per_mm)
                     self.hull_dict['{}_hull_mean_euclidean'.format(animal_name)].append(np.mean(animal) / self.px_per_mm)
                     self.hull_dict['{}_hull_sum_euclidean'.format(animal_name)].append(np.sum(animal, initial=0) / self.px_per_mm)
             for k, v in self.hull_dict.items():
@@ -390,8 +323,8 @@ class ExtractFeaturesFrom16bps(object):
                 self.out_data[currentColName_M1] = self.out_data[tail_end_col_name] - (self.out_data[tail_base_col_name] + self.out_data[centroid_col_name] + self.out_data[nose_col_name])
                 self.out_data[currentColName_M2] = self.out_data[tail_end_col_name_M2] - (self.out_data[tail_base_col_name_M2] + self.out_data[centroid_col_name_M2] + self.out_data[nose_col_name_M2])
 
-            self.out_data['Mouse_1_angle'] = self.__angle3pt_serialized(data=self.out_data[['Nose_1_x', 'Nose_1_y', 'Center_1_x', 'Center_1_y', 'Tail_base_1_x', 'Tail_base_1_y']].values)
-            self.out_data['Mouse_2_angle'] = self.__angle3pt_serialized(data=self.out_data[['Nose_2_x', 'Nose_2_y', 'Center_2_x', 'Center_2_y', 'Tail_base_2_x', 'Tail_base_2_y']].values)
+            self.out_data['Mouse_1_angle'] = self.angle3pt_serialized(data=self.out_data[['Nose_1_x', 'Nose_1_y', 'Center_1_x', 'Center_1_y', 'Tail_base_1_x', 'Tail_base_1_y']].values)
+            self.out_data['Mouse_2_angle'] = self.angle3pt_serialized(data=self.out_data[['Nose_2_x', 'Nose_2_y', 'Center_2_x', 'Center_2_y', 'Tail_base_2_x', 'Tail_base_2_y']].values)
             self.out_data['Total_angle_both_mice'] = self.out_data['Mouse_1_angle'] + self.out_data['Mouse_2_angle']
             for window in self.roll_windows_values:
                 currentColName = 'Total_angle_both_mice_{}'.format(str(window))
@@ -453,7 +386,7 @@ class ExtractFeaturesFrom16bps(object):
             self.out_data['Movement_percentile_rank'] = self.out_data['Total_movement_centroids'].rank(pct=True)
             self.out_data['Distance_percentile_rank'] = self.out_data['Centroid_distance'].rank(pct=True)
             self.out_data['Movement_mouse_1_percentile_rank'] = self.out_data['Movement_mouse_1_centroid'].rank(pct=True)
-            self.out_data['Movement_mouse_2_percentile_rank'] = self.out_data['Movement_mouse_1_centroid'].rank(pct=True)
+            self.out_data['Movement_mouse_2_percentile_rank'] = self.out_data['Movement_mouse_2_centroid'].rank(pct=True)
             self.out_data['Movement_mouse_1_deviation_percentile_rank'] = self.out_data['Movement_mouse_1_deviation_centroid'].rank(pct=True)
             self.out_data['Movement_mouse_2_deviation_percentile_rank'] = self.out_data['Movement_mouse_2_deviation_centroid'].rank(pct=True)
             self.out_data['Centroid_distance_percentile_rank'] = self.out_data['Centroid_distance'].rank(pct=True)
@@ -510,13 +443,13 @@ class ExtractFeaturesFrom16bps(object):
                     c_centroid_lst_mouse1_x, c_centroid_lst_mouse1_y = centroid_lst_mouse1_x[start:end], centroid_lst_mouse1_y[start:end]
                     c_centroid_lst_mouse2_x, c_centroid_lst_mouse2_y = centroid_lst_mouse2_x[start:end], centroid_lst_mouse2_y[start:end]
                     for frame_in_window in range(len(c_centroid_lst_mouse1_x)):
-                        move_angle_mouse_1 = (self.__angle3pt(c_centroid_lst_mouse1_x[frame_in_window][0],
+                        move_angle_mouse_1 = (self.angle3pt(c_centroid_lst_mouse1_x[frame_in_window][0],
                                                             c_centroid_lst_mouse1_y[frame_in_window][0],
                                                             c_centroid_lst_mouse1_x[frame_in_window][1],
                                                             c_centroid_lst_mouse1_y[frame_in_window][1],
                                                             c_centroid_lst_mouse1_x[frame_in_window][2],
                                                             c_centroid_lst_mouse1_y[frame_in_window][2]))
-                        move_angle_mouse_2 = (self.__angle3pt(c_centroid_lst_mouse2_x[frame_in_window][0],
+                        move_angle_mouse_2 = (self.angle3pt(c_centroid_lst_mouse2_x[frame_in_window][0],
                                                             c_centroid_lst_mouse2_y[frame_in_window][0],
                                                             c_centroid_lst_mouse2_x[frame_in_window][1],
                                                             c_centroid_lst_mouse2_y[frame_in_window][1],
@@ -537,14 +470,14 @@ class ExtractFeaturesFrom16bps(object):
             self.out_data['Sum_probabilities_deviation'] = (self.out_data['Sum_probabilities'].mean() - self.out_data['Sum_probabilities'])
             self.out_data['Sum_probabilities_deviation_percentile_rank'] = self.out_data['Sum_probabilities_deviation'].rank(pct=True)
             self.out_data['Sum_probabilities_percentile_rank'] = self.out_data['Sum_probabilities_deviation_percentile_rank'].rank(pct=True)
-            results = pd.DataFrame(self.__count_values_in_range(data=self.out_data.filter(all_p_columns).values, ranges=np.array([[0.0, 0.1], [0.000000000, 0.5], [0.000000000, 0.75]])), columns=['Low_prob_detections_0.1', 'Low_prob_detections_0.5', 'Low_prob_detections_0.75'])
+            results = pd.DataFrame(self.count_values_in_range(data=self.out_data.filter(all_p_columns).values, ranges=np.array([[0.0, 0.1], [0.000000000, 0.5], [0.000000000, 0.75]])), columns=['Low_prob_detections_0.1', 'Low_prob_detections_0.5', 'Low_prob_detections_0.75'])
             self.out_data = pd.concat([self.out_data, results], axis=1)
 
             self.out_data = self.out_data.reset_index(drop=True).fillna(0)
             save_path = os.path.join(self.save_dir, self.video_name + '.' + self.file_type)
             save_df(self.out_data, self.file_type, save_path)
             session_time, file_time = session_time + (time.time()-file_start_time), int(time.time() - file_start_time)
-            print('Feature extraction complete for {} ({}/{} (elapsed time: {}s))...'.format(self.video_name, str(file_cnt + 1), str(len(self.files_found)), str(file_time)))
+            print('Feature extraction complete for {} ({}/{} (elapsed time: {}s)...'.format(self.video_name, str(file_cnt + 1), str(len(self.files_found)), str(file_time)))
 
         print('SIMBA COMPLETE: All features extracted (elapsed time: {}s). Results stored in project_folder/csv/features_extracted directory'.format(str(int(session_time))))
 
