@@ -4,6 +4,7 @@ from datetime import datetime
 import warnings
 from tables import NaturalNameWarning
 from simba.misc_tools import SimbaTimer
+from simba.read_config_unit_tests import check_if_filepath_list_is_empty
 warnings.filterwarnings('ignore', category=NaturalNameWarning)
 
 class KeypointRemover(object):
@@ -38,12 +39,11 @@ class KeypointRemover(object):
             print('SIMBA ERROR: {} is not a valid directory.'.format(str(data_folder)))
             raise NotADirectoryError
         self.files_found = glob.glob(data_folder + '/*.' + file_format)
+        check_if_filepath_list_is_empty(filepaths=self.files_found,
+                                        error_msg='SIMBA ERROR: Zero files found of type {} in the {} directory'.format(file_format, data_folder))
         self.datetime = str(datetime.now().strftime('%Y%m%d%H%M%S'))
         self.pose_tool, self.data_folder = pose_tool, data_folder
         self.file_format = file_format
-        if len(self.files_found) < 1:
-            print('SIMBA ERROR: Zero files found of type {} in the {} directory'.format(file_format, data_folder))
-            raise ValueError
         if file_format == 'h5':
             first_df = pd.read_hdf(self.files_found[0])
         else:
