@@ -1,4 +1,4 @@
-__author__ = "Simon Nilsson"
+__author__ = "Simon Nilsson, JJ Choong"
 
 import itertools
 from configparser import ConfigParser, MissingSectionHeaderError
@@ -14,10 +14,12 @@ from simba.drop_bp_cords import (get_workflow_file_format,
 from simba.misc_tools import (find_video_of_file,
                               check_multi_animal_status,
                               smooth_data_gaussian,
-                              smooth_data_savitzky_golay)
-
-from simba.misc_tools import get_video_meta_data
-
+                              smooth_data_savitzky_golay,
+                              get_video_meta_data)
+from simba.read_config_unit_tests import (check_if_filepath_list_is_empty,
+                                          read_project_path_and_file_type,
+                                          read_config_file)
+from simba.enums import Paths, ReadConfig, Methods
 from simba.rw_dfs import read_df
 import json
 from collections import defaultdict
@@ -40,7 +42,7 @@ class ImportSLEAP(object):
     data_folder: str
         Path to folder containing SLEAP data in `.slp` format.
     actor_IDs: list
-        List of animal names.
+        Animal names.
     interpolation_settings: str
         String defining the pose-estimation interpolation method. OPTIONS: 'None', 'Animal(s): Nearest',
         'Animal(s): Linear', 'Animal(s): Quadratic','Body-parts: Nearest', 'Body-parts: Linear',
@@ -66,29 +68,23 @@ class ImportSLEAP(object):
                  interpolation_settings: str,
                  smoothing_settings: dict):
 
-
         self.ini_path = project_path
         self.interpolation_settings = interpolation_settings
         self.smoothing_settings = smoothing_settings
         self.config = ConfigParser()
         self.actors_IDs = actor_IDs
-        try:
-            self.config.read(project_path)
-        except MissingSectionHeaderError:
-            print('ERROR:  Not a valid project_config file. Please check the project_config.ini path.')
-        self.project_path = self.config.get('General settings', 'project_path')
+        self.config = read_config_file(ini_path=project_path)
+        self.project_path, self.file_format = read_project_path_and_file_type(config=self.config)
         self.video_folder = os.path.join(self.project_path, 'videos')
         self.files_found = glob.glob(data_folder + '/*.slp')
-        if len(self.files_found) == 0:
-            print('SIMBA ERROR: Zero .slp files found in {} directory'.format(data_folder))
-            raise ValueError()
-        self.save_folder = os.path.join(self.project_path, 'csv', 'input_csv')
-        self.file_format = get_workflow_file_format(self.config)
+        check_if_filepath_list_is_empty(filepaths=self.files_found,
+                                        error_msg='SIMBA ERROR: Zero .slp files found in {} directory'.format(data_folder))
+        self.save_folder = os.path.join(self.project_path, Paths.INPUT_CSV.value)
         self.animals_no = len(self.actors_IDs)
         self.add_spacer = 2
-        self.bp_names_csv_path = os.path.join(self.project_path, 'logs', 'measures', 'pose_configs', 'bp_names', 'project_bp_names.csv')
-        self.pose_settings = self.config.get('create ensemble settings', 'pose_estimation_body_parts')
-        if self.pose_settings is 'user_defined':
+        self.bp_names_csv_path = os.path.join(self.project_path, Paths.BP_NAMES.value)
+        self.pose_settings = self.config.get(ReadConfig.CREATE_ENSEMBLE_SETTINGS.value, ReadConfig.POSE_SETTING.value)
+        if self.pose_settings is Methods.USER_DEFINED.value:
             self.__update_config()
 
         print('Converting .SLP file(s) into SimBA dataframes...')
@@ -98,8 +94,7 @@ class ImportSLEAP(object):
         multi_animal_status, multi_animal_id_lst = check_multi_animal_status(self.config, self.animals_no)
         Xcols, Ycols, Pcols = getBpNames(self.ini_path)
         color_lst = createColorListofList(self.animals_no, len(self.analysis_dict['ordered_bps']))
-        self.animal_bp_dict = create_body_part_dictionary(multi_animal_status, multi_animal_id_lst, self.animals_no,
-                                                          Xcols, Ycols, [], color_lst)
+        self.animal_bp_dict = create_body_part_dictionary(multi_animal_status, multi_animal_id_lst, self.animals_no, Xcols, Ycols, [], color_lst)
 
         for file_path in self.save_paths_lst:
             self.data_df = read_df(file_path, self.file_format)
@@ -152,7 +147,6 @@ class ImportSLEAP(object):
         self.data_df = pd.concat([self.data_df, missing_df], axis=0)
 
     def __save_multi_index_header_df(self, df=None, filetype=None, savepath=None):
-        self.save_paths_lst = []
         if filetype == 'csv':
             df.to_csv(savepath)
         if filetype == 'parquet':
@@ -184,6 +178,7 @@ class ImportSLEAP(object):
 
         """
         self.analysis_dict = defaultdict(list)
+        self.save_paths_lst = []
         for vdn_cnt, file_path in enumerate(self.files_found):
             print('Analysing {}{}'.format(os.path.basename(file_path), '...'))
             self.file_path = file_path
@@ -235,13 +230,12 @@ class ImportSLEAP(object):
         for frame_cnt, frame in enumerate(range(self.analysis_dict['no_frames'])):
             frame_idx = self.analysis_dict['frames'][frame_cnt][2]
             self.frame_dict = {}
-            print('Frame: {}/{}, Video: {} ({}/{})'.format(str(frame_cnt), str(self.analysis_dict['no_frames']), str(self.video_name), str(self.video_counter + 1), str(len(self.files_found))))
+            print('Restructuring SLEAP frame: {}/{}, Video: {} ({}/{})'.format(str(frame_cnt), str(self.analysis_dict['no_frames']), str(self.video_name), str(self.video_counter + 1), str(len(self.files_found))))
             self.cnt_animals_frm = self.analysis_dict['animals_in_each_frame'][frame]
             if self.cnt_animals_frm == 0:
                 self.frame_dict[0] = [0] * len(self.analysis_dict['xyp_headers'])
                 end_frame = start_frame + (len(self.analysis_dict['ordered_bps']) * self.cnt_animals_frm)
 
-
             else:
                 end_frame = start_frame + (len(self.analysis_dict['ordered_bps']) * self.cnt_animals_frm)
                 start_animal, end_animal = 0, len(self.analysis_dict['ordered_bps'])
@@ -264,6 +258,7 @@ class ImportSLEAP(object):
                 self.data_df.loc[frame_idx] = frame_lst
             except ValueError:
                 break
+
         self.data_df.fillna(0, inplace=True)
         self.__fill_missing_indexes()
         self.data_df.sort_index(inplace=True)
@@ -467,6 +462,32 @@ class ImportSLEAP(object):
 # test.perform_smothing()
 # print('All SLEAP imports complete.')
 #
-#
+
+# test = ImportSLEAP(project_path="/Users/simon/Desktop/envs/troubleshooting/sleap_5_animals/project_folder/project_config.ini",
+#                    data_folder=r'/Users/simon/Desktop/envs/troubleshooting/sleap_5_animals/data',
+#                    actor_IDs=['Simon', 'Nastacia', 'JJ', 'Sam', 'Liana'],
+#                    interpolation_settings="Body-parts: Nearest",
+#                    smoothing_settings = {'Method': 'Savitzky Golay', 'Parameters': {'Time_window': '200'}}) #Savitzky Golay
+# test.initate_import_slp()
+# if test.animals_no > 1:
+#     test.visualize_sleap()
+# test.save_df()
+# test.perform_interpolation()
+# test.perform_smothing()
+# print('All SLEAP imports complete.')
+
+
+# test = ImportSLEAP(project_path="/Users/simon/Desktop/envs/troubleshooting/sleap_cody/project_folder/project_config.ini",
+#                    data_folder=r'/Users/simon/Desktop/envs/troubleshooting/sleap_cody/import_data',
+#                    actor_IDs=['Animal_1', 'Animal_2'],
+#                    interpolation_settings="Body-parts: Nearest",
+#                    smoothing_settings = {'Method': 'Savitzky Golay', 'Parameters': {'Time_window': '200'}}) #Savitzky Golay
+# test.initate_import_slp()
+# if test.animals_no > 1:
+#     test.visualize_sleap()
+# test.save_df()
+# test.perform_interpolation()
+# test.perform_smothing()
+# print('All SLEAP imports complete.')
 
 
