@@ -233,8 +233,9 @@ class TrainSingleModel(object):
                                  generate_example_decision_tree, generate_features_importance_bar_graph,
                                  generate_features_importance_log, generate_precision_recall_curve, save_meta_data,
                                  generate_learning_curve, dataset_splits, shuffle_splits, feature_importance_bars,
-                                 self.over_sample_ratio, self.over_sample_setting, self.tt_size,
-                                 self.under_sample_ratio, self.under_sample_ratio, str(class_weights)]
+                                 self.over_sample_ratio, self.over_sample_setting, self.tt_size, self.split_type,
+                                 self.under_sample_ratio, self.under_sample_setting, str(class_weights)]
+
                 create_meta_data_csv_training_one_model(meta_data_lst, self.clf_name, self.eval_out_path)
 
     def save_model(self):
@@ -252,7 +253,7 @@ class TrainSingleModel(object):
         print('Classifier {} saved in models/generated_models directory (elapsed time: {}s).'.format(self.clf_name, self.timer.elapsed_time_str))
         print('Evaluation files are in models/generated_models/model_evaluations folders')
 
-# test = TrainSingleModel(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')
+# test = TrainSingleModel(config_path='/Users/simon/Desktop/envs/troubleshooting/Two_animals_16bps/project_folder/project_config.ini')
 # test.perform_sampling()
 # test.train_model()
 # test.save_model()
