@@ -5,6 +5,7 @@ import numpy as np
 import imutils
 from simba.read_config_unit_tests import check_file_exist_and_readable
 from simba.drop_bp_cords import createColorListofList
+from simba.enums import Paths
 import simba
 import os, glob
 
@@ -24,9 +25,9 @@ class PoseConfigCreator(object):
     img_path: str
         Path to image representation of user-defined pose-estimation setting
     bp_list: list
-        List of body-parts in the user-defined pose-estimation setting
+        Body-parts in the user-defined pose-estimation setting
     animal_id_int_list: list
-        List of integers representing the animal ID which the body-parts belong to
+        Integers representing the animal ID which the body-parts belong to
 
     Notes
     ----------
@@ -71,6 +72,7 @@ class PoseConfigCreator(object):
                 self.bp_list[cnt] = '{}_{}'.format(bp_name, animal_number_id)
         self.color_lst = createColorListofList(1, len(self.bp_list))[0]
 
+
     def launch(self):
         def draw_circle(event, x, y, flags, param):
             if (event == cv2.EVENT_LBUTTONDBLCLK):
@@ -84,10 +86,12 @@ class PoseConfigCreator(object):
             self.side_img = np.zeros((int(self.img_height/4), self.img_width, 3), np.uint8)
             cv2.putText(self.side_img, 'Double left click on body part {}.'.format(bp_name), (10, 50), self.font, self.font_size, self.color_lst[bp_cnt], 2)
             img_concat = cv2.vconcat([self.side_img, self.overlay])
+            cv2.namedWindow('Define pose', cv2.WINDOW_NORMAL)
             cv2.imshow('Define pose', img_concat)
             while not self.cord_written:
                 cv2.setMouseCallback('Define pose', draw_circle)
                 img_concat = cv2.vconcat([self.side_img, self.overlay])
+                cv2.namedWindow('Define pose', cv2.WINDOW_NORMAL)
                 cv2.imshow('Define pose', img_concat)
                 cv2.waitKey(1)
             cv2.destroyWindow('Define pose')
@@ -96,10 +100,10 @@ class PoseConfigCreator(object):
     def save(self):
         overlay = cv2.resize(self.overlay, (250, 300))
         simba_cw = os.path.dirname(simba.__file__)
-        img_dir = os.path.join(simba_cw, 'pose_configurations', 'schematics')
-        pose_name_path = os.path.join(simba_cw, 'pose_configurations', 'configuration_names', 'pose_config_names.csv')
-        bp_path = os.path.join(simba_cw, 'pose_configurations', 'bp_names', 'bp_names.csv')
-        no_animals_path = os.path.join(simba_cw, 'pose_configurations', 'no_animals', 'no_animals.csv')
+        img_dir = os.path.join(simba_cw, Paths.SCHEMATICS.value)
+        pose_name_path = os.path.join(simba_cw, Paths.PROJECT_POSE_CONFIG_NAMES.value)
+        bp_path = os.path.join(simba_cw, Paths.SIMBA_BP_CONFIG_PATH.value)
+        no_animals_path = os.path.join(simba_cw, Paths.SIMBA_NO_ANIMALS_PATH.value)
         for path in [pose_name_path, bp_path, no_animals_path]:
             check_file_exist_and_readable(file_path=path)
         prior_imgs = len(glob.glob(img_dir + '/*.png'))
